from stumpf.diva_fact_schemas import ActivityFact, ActivityFactSchema
from stumpf.models import Assignment, Task

activity_fact_schema = ActivityFactSchema()
MINIMUM_OVERLAP = 0.5


def app_context():
    from stumpf.app import create_app
    app = create_app({'SQLALCHEMY_ECHO': False})
    return app.app_context()


def get_ground_truth(task):
    """TODO: Find the correct ground truth fact for an assignment."""
    activity_id = task.args.get('activityId')
    if activity_id == '98c582eb-74a6-49ac-8149-08a73486e7c8':
        fact = ActivityFact.query.get('ce22c568-a386-49b9-aaee-db4c22112c0b')
    elif activity_id == 'ff6156a4-cc71-4a12-83e1-88a23413ab96':
        fact = ActivityFact.query.get('7f4e06c8-e132-4d4e-89a9-d34f722f0307')

    return activity_fact_schema.dump(fact)


def get_assignments(task_id):
    query = Assignment.query
    query = query.filter_by(task_id=task_id)
    query = query.filter(Assignment.status.in_(['Approved', 'Submitted']))
    query = query.filter(Assignment.answer != None)  # noqa
    return query.all()


def convert_answer_detection_to_fact_object(detection):
    bbox = detection['bbox']
    return {
        'frame': detection['frame'],
        'left': bbox[0],
        'right': bbox[2],
        'top': bbox[1],
        'bottom': bbox[3],
        'source': detection['src']
    }


def generate_answer_tracks(answer):
    tracks = []
    for detection in answer['detections']:
        id_ = detection['id1'] - 1
        if id_ >= len(tracks):
            tracks.append({
                'detections': [],
                'keyframes': []
            })
        converted_detection = convert_answer_detection_to_fact_object(detection)
        tracks[id_]['detections'].append(converted_detection)
        if converted_detection['source'] == 'truth':
            tracks[id_]['keyframes'].append(converted_detection)

    # sort the detection lists
    for track in tracks:
        track['detections'] = sorted(track['detections'], key=lambda d: d['frame'])
        track['keyframes'] = sorted(track['keyframes'], key=lambda d: d['frame'])

    return tracks


def frame_matches(frame1, frame2):
    return frame1['frame'] == frame2['frame'] and \
        frame1['left'] == frame2['left'] and \
        frame1['right'] == frame2['right'] and \
        frame1['top'] == frame2['top'] and \
        frame1['bottom'] == frame2['bottom']


def find_matching_answer_track(answer_tracks, detections):
    first_frame = detections[0]
    for track in answer_tracks:
        answer_frame = track['detections'][0]
        if frame_matches(first_frame, answer_frame):
            break
    else:
        raise Exception('Could not find matching track in assignment answer')
    return track


def detection_area(detection):
    if 'area' not in detection:
        detection['area'] = \
            (detection['right'] - detection['left']) * (detection['bottom'] - detection['top'])
    return detection['area']


def intersection(detection1, detection2):
    left = max(detection1['left'], detection2['left'])
    right = min(detection1['right'], detection2['right'])
    top = max(detection1['top'], detection2['top'])
    bottom = min(detection1['bottom'], detection2['bottom'])
    return {
        'left': left,
        'right': max(left, right),
        'top': top,
        'bottom': max(top, bottom)
    }


def intersection_over_union(detection1, detection2):
    intersection_area = detection_area(intersection(detection1, detection2))
    union_area = detection_area(detection1) + detection_area(detection2) - intersection_area
    if union_area == 0:
        return 1
    return intersection_area / union_area


def score_track(answer_tracks, detections):
    answer_track = find_matching_answer_track(answer_tracks, detections)
    total_score = 0
    number_of_frames = len(detections)
    assert number_of_frames == len(answer_track['detections'])

    for i in range(number_of_frames):
        # for iou metric:
        # total_score += intersection_over_union(detections[i], answer_track['detections'][i])
        # for grouper paper metric:
        total_score += total_grouper_score(detections[i], answer_track['detections'][i])

    return total_score / number_of_frames


def score_assignment(assignment, truth):
    answer_tracks = generate_answer_tracks(assignment.answer)
    total_score = 0
    for track in truth['tracks']:
        detections = sorted(track['detections'], key=lambda d: d['frame'])
        total_score += score_track(answer_tracks, detections)
    return total_score / len(truth['tracks'])


# Metrics described by:
# https://goo.gl/1KEpGG
def overlap_score(box1, box2):
    area1 = detection_area(box1)
    area2 = detection_area(box2)
    largest_area = max(area1, area2)
    overlap_area = detection_area(intersection(box1, box2))
    if largest_area == 0:
        return 0
    return overlap_area / largest_area


def size_score(box1, box2):
    area1 = detection_area(box1)
    area2 = detection_area(box2)
    min_area = min(area1, area2)
    max_area = max(area1, area2)
    return 1 - (1 - min_area / max_area) / (1 - MINIMUM_OVERLAP)


def box_ratio(box):
    return (box['right'] - box['left']) / (box['bottom'] - box['top'])


def shape_score(box1, box2):
    ratio1 = box_ratio(box1)
    ratio2 = box_ratio(box2)
    min_ratio = min(ratio1, ratio2)
    max_ratio = max(ratio1, ratio2)
    return 1 - (1 - min_ratio / max_ratio) / (1 - MINIMUM_OVERLAP * MINIMUM_OVERLAP)


def box_center(box):
    return (box['left'] + box['right']) / 2, (box['top'] + box['bottom']) / 2


def position_score(box1, box2):
    center_x1, center_y1 = box_center(box1)
    center_x2, center_y2 = box_center(box2)
    delta_x = abs(center_x1 - center_x2)
    delta_y = abs(center_y1 - center_y2)
    max_width = max(box1['right'] - box1['left'], box2['right'] - box2['left'])
    max_height = max(box1['bottom'] - box1['top'], box2['bottom'] - box2['top'])
    distance_width_ratio = delta_x / max_width
    distance_height_ratio = delta_y / max_height
    return 1 - (distance_width_ratio + distance_height_ratio) / (2 * (1 - MINIMUM_OVERLAP))


def total_grouper_score(box1, box2):
    overlap = overlap_score(box1, box2)
    if overlap < MINIMUM_OVERLAP:
        return overlap / 4

    size = size_score(box1, box2)
    shape = shape_score(box1, box2)
    position = position_score(box1, box2)
    return (overlap + size + shape + position) / 4


def main(task_id):
    with app_context():
        task = Task.query.get(task_id)
        truth = get_ground_truth(task)
        for assignment in get_assignments(task_id):
            score = score_assignment(assignment, truth)
            url = ('https://matthewma7.github.io/refiner_dev/refiner'
                   f'/result/?assignmentId={str(assignment.id)}')
            print('%6.4f - %s' % (score, url))


if __name__ == '__main__':
    import sys
    main(sys.argv[1])
