import json
import logging
import os
from shutil import copyfileobj
import time
from urllib.request import urlopen

import click
import hvac
from stumpf.app import create_app
from stumpf.models import db, Fact, Provenance
from stumpf.diva_fact_schemas import GladiatorFact, ActivityFact

from stumpfwald import authorized, cli

GLADIATOR_APP_ID = 'd4f8d27a-b3b6-44c3-b6ed-1e8910048c2d'


def get_production_database_uri(vault_token):
    client = hvac.Client(url='https://vault.kitware.com', token=vault_token)
    db_creds = client.read('secret/data/projects/diva-mturk/stumpf/rds')['data']['data']
    return f"postgresql://{db_creds['username']}:{db_creds['password']}@{db_creds['host']}/{db_creds['dbname']}"  # noqa


@click.group(name='gladiator', short_help='The Grozper related commands.')
@click.option('--production', is_flag=True, default=False,
              help='Use the production Stumpf database.')
@click.option('--vault-token', type=click.UUID, help='The token to use to authenticate to Vault.')
@click.pass_obj
def gladiator(ctx, production, vault_token):
    logging.getLogger('flask.app').setLevel(logging.ERROR)
    ctx['production'] = production
    app_config = {
        'SQLALCHEMY_ECHO': False,
    }
    if ctx['production']:
        app_config['SQLALCHEMY_DATABASE_URI'] = get_production_database_uri(vault_token)

    ctx['app'] = create_app(app_config)


@gladiator.command(name='hit-from-grozper', help='Create Gladiator HITs from Grozper fact')
@click.option('--after', type=click.DateTime())
@click.option('--before', type=click.DateTime())
@click.option('--batch', type=click.STRING)
@click.argument('activity-ids', nargs=-1, type=click.UUID)
@click.option('--sandbox', type=click.BOOL, default=False)
@click.option('--dry-run', is_flag=True)
@click.pass_obj
@authorized
def create_hits_from_grozper_fact(ctx, after, before, batch, activity_ids, sandbox, dry_run):
    if not after and not batch and not activity_ids:
        raise click.UsageError('--after, --batch or activity-ids is required')

    with ctx['app'].app_context():
        if after:
            query = ActivityFact.query.filter(ActivityFact.created >= after)
            if before:
                query = query.filter(ActivityFact.created < before)
        elif batch:
            sub = db.session.query(Provenance.fact_id).join(
                ActivityFact).filter(Provenance.meta['batch'].astext == batch)
            query = ActivityFact.query.filter(ActivityFact.id.in_(sub))
        elif activity_ids:
            query = ActivityFact.query.filter(ActivityFact.id.in_(activity_ids))

        if dry_run:
            click.echo(f'{query.count()} Gladiator HITs would be created')
            return

        for activityFact in query:
            r = ctx['session'].post(f'task/{activityFact.video_id}/{GLADIATOR_APP_ID}/hit', json={
                'sandbox': sandbox,
                'title': 'Validate video annotation quality',
                'description': 'approve with a yes or deny with a no',
                'assignment_duration_in_seconds': 1800,
                'max_assignments': 3,
                'reward':  5,
                'lifetime_in_seconds': 24 * 3600,
                'args': {
                    'activtyId': str(activityFact.id)
                },
                'qualification_requirements': [
                    {
                        "QualificationTypeId": "00000000000000000040",
                        "Comparator": "GreaterThan",
                        "IntegerValues": [1000],
                        "ActionsGuarded": "PreviewAndAccept"
                    }, {
                        "QualificationTypeId": "000000000000000000L0",
                        "Comparator": "GreaterThan",
                        "IntegerValues": [95],
                        "ActionsGuarded": "PreviewAndAccept"
                    }
                ]
            })
            if not r.ok:
                click.echo(r.text, err=True)
            else:
                click.echo(json.dumps(r.json(), indent=4))
            time.sleep(2)


cli.add_command(gladiator)
