import json
import logging
import os
import sys
from uuid import uuid4

import click
from stumpf.app import create_app
from stumpf.models import db, Video

from stumpfwald import cli
from stumpfwald.importers import grozper_activity_importer
from stumpfwald.kw2json import kw2json
from stumpfwald.refiner import get_production_database_uri


@click.group(name='grozper', short_help='The Grozper related commands.')
@click.option('--production', is_flag=True, default=False,
              help='Use the production Stumpf database.')
@click.option('--vault-token', type=click.UUID, help='The token to use to authenticate to Vault.')
@click.pass_obj
def grozper(ctx, production, vault_token):
    logging.getLogger('flask.app').setLevel(logging.ERROR)
    ctx['production'] = production
    app_config = {
        'SQLALCHEMY_ECHO': False,
    }
    if ctx['production']:
        app_config['SQLALCHEMY_DATABASE_URI'] = get_production_database_uri(vault_token)

    ctx['app'] = create_app(app_config)


@grozper.command(name='import-json',
                 help='Import activities, tracks, and detections from Grouper JSON.')
@click.argument('video-id', type=click.UUID)
@click.argument('infiles', nargs=-1, type=click.File(), metavar='FILE')
@click.option('--batch', nargs=1, default=str(uuid4()))
@click.pass_obj
def import_grozper_json(ctx, video_id, infiles, batch):
    with ctx['app'].app_context():
        video = Video.query.get(video_id)
        if not video:
            click.echo(f'Video with ID {video_id} does not exist.', err=True)
            sys.exit(1)

        for infile in infiles:
            activity = json.load(infile)
            grozper_activity_importer.import_activity_group(activity, video, {
                'batch': batch,
                'stumpfwald': True,
                'file': infile.name
            })
        db.session.commit()


@grozper.command(name='import-kw18',
                 help='Import activities, tracks, and detections from Grouper kw18 project.')
@click.argument('video-id', type=click.UUID)
@click.argument('project-path', type=click.Path(exists=True), metavar='PATH')
@click.option('--batch', nargs=1, default=str(uuid4()))
@click.pass_obj
def import_grozper_kw18(ctx, video_id, project_path, batch):
    with ctx['app'].app_context():
        video = Video.query.get(video_id)
        if not video:
            click.echo(f'Video with ID {video_id} does not exist.', err=True)
            sys.exit(1)

        for file in os.listdir(project_path):
            if file.endswith('.kw18'):
                kw18_fbase = os.path.splitext(file)[0]
                break

        if not kw18_fbase:
            click.echo(f'Didn\'t find .kw18 file in directory {kw18_fbase}.', err=True)
            sys.exit(1)

        provenance = {
            'batch': batch,
            'stumpfwald': True,
            'project_name': kw18_fbase
        }

        for activity in kw2json(project_path, kw18_fbase):
            grozper_activity_importer.import_activity_group(activity, video, provenance)
        db.session.commit()


cli.add_command(grozper)
